@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2002, 2004, 2005 Red Hat, Inc.
+ * Copyright (C) 2000-2002, 2004, 2005, 2006 Red Hat, Inc.
  *
  * This is free software; you can redistribute it and/or modify it under
  * the terms of the GNU Library General Public License as published by
@@ -16,7 +16,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#ident "$Id: apputil.c,v 1.39 2005/09/12 23:24:36 mitr Exp $"
+#ident "$Id: apputil.c,v 1.44 2006/05/01 04:51:21 mitr Exp $"
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -25,7 +25,6 @@
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <sys/time.h>
-#include <ctype.h>
 #include <dirent.h>
 #include <fcntl.h>
 #include <glib.h>
@@ -104,50 +103,49 @@ setup_default_context(const char *orig_file)
 }
 #endif
 
-/* Populate a user's home directory, copying data from a named skeleton
- * directory, setting all ownerships as given, and setting the mode of
- * the top-level directory as given. */
-gboolean
-lu_homedir_populate(const char *skeleton, const char *directory,
-		    uid_t owner, gid_t group, mode_t mode,
-		    struct lu_error **error)
+/* Copy the "src" directory to "dest", setting all ownerships as given, and
+   setting the mode of the top-level directory as given.  The group ID of the
+   copied files is preserved if it is nonzero. */
+static gboolean
+lu_homedir_copy(const char *src, const char *dest, uid_t owner, gid_t group,
+		mode_t mode, struct lu_error **error)
 {
 	struct dirent *ent;
 	DIR *dir;
-	struct stat st;
-	char skelpath[PATH_MAX], path[PATH_MAX], buf[PATH_MAX];
-	struct utimbuf timebuf;
-	int ifd = -1, ofd = -1;
-	off_t offset;
+	int ifd, ofd;
 	gboolean ret = FALSE;
 
 	LU_ERROR_CHECK(error);
 
-	dir = opendir(skeleton);
+	dir = opendir(src);
 	if (dir == NULL) {
 		lu_error_new(error, lu_error_generic,
-			     _("Error reading `%s': %s"), skeleton,
+			     _("Error reading `%s': %s"), src,
 			     strerror(errno));
 		goto err;
 	}
 
 	/* Create the top-level directory. */
-	if ((mkdir(directory, mode) == -1) && (errno != EEXIST)) {
+	if (mkdir(dest, mode) == -1 && errno != EEXIST) {
 		lu_error_new(error, lu_error_generic,
-			     _("Error creating `%s': %s"), directory,
+			     _("Error creating `%s': %s"), dest,
 			     strerror(errno));
 		goto err_dir;
 	}
 
 	/* Set the ownership on the top-level directory. */
-	if (chown(directory, owner, group) == -1 && errno != EPERM) {
+	if (chown(dest, owner, group) == -1 && errno != EPERM) {
 		lu_error_new(error, lu_error_generic,
-			     _("Error changing owner of `%s': %s"), directory,
+			     _("Error changing owner of `%s': %s"), dest,
 			     strerror(errno));
 		goto err_dir;
 	}
 
 	while ((ent = readdir(dir)) != NULL) {
+		char srcpath[PATH_MAX], path[PATH_MAX], buf[PATH_MAX];
+		struct stat st;
+		struct utimbuf timebuf;
+
 		/* Iterate through each item in the directory. */
 		/* Skip over self and parent hard links. */
 		if (strcmp(ent->d_name, ".") == 0) {
@@ -157,163 +155,153 @@ lu_homedir_populate(const char *skeleton, const char *directory,
 			continue;
 		}
 
-		/* Build the path of the skeleton file or directory and
-		 * its corresponding member in the new tree. */
-		snprintf(skelpath, sizeof(skelpath), "%s/%s",
-			 skeleton, ent->d_name);
-		snprintf(path, sizeof(path), "%s/%s", directory,
-			 ent->d_name);
+		/* Build the path of the source file or directory and its
+		   corresponding member in the new tree. */
+		snprintf(srcpath, sizeof(srcpath), "%s/%s", src, ent->d_name);
+		snprintf(path, sizeof(path), "%s/%s", dest, ent->d_name);
 
 		/* What we do next depends on the type of entry we're
 		 * looking at. */
-		if (lstat(skelpath, &st) != -1) {
-			/* We always want to preserve atime/mtime. */
-			timebuf.actime = st.st_atime;
-			timebuf.modtime = st.st_mtime;
+		if (lstat(srcpath, &st) != 0)
+			continue;
 
-			/* If it's a directory, descend into it. */
-			if (S_ISDIR(st.st_mode)) {
-				if (!lu_homedir_populate(skelpath,
-							 path,
-							 owner,
-							 st.st_gid ?: group,
-							 st.st_mode,
-							 error)) {
-					/* Aargh!  Fail up. */
-					goto err_dir;
+		/* We always want to preserve atime/mtime. */
+		timebuf.actime = st.st_atime;
+		timebuf.modtime = st.st_mtime;
+
+		/* If it's a directory, descend into it. */
+		if (S_ISDIR(st.st_mode)) {
+			if (!lu_homedir_copy(srcpath, path, owner,
+					     st.st_gid ?: group, st.st_mode,
+					     error))
+				/* Aargh!  Fail up. */
+				goto err_dir;
+			/* Set the date on the directory. */
+			utime(path, &timebuf);
+			continue;
+		}
+
+		/* If it's a symlink, duplicate it. */
+		if (S_ISLNK(st.st_mode)) {
+			ssize_t len;
+
+			len = readlink(srcpath, buf, sizeof(buf) - 1);
+			if (len == -1) {
+				lu_error_new(error, lu_error_generic,
+					     _("Error reading `%s': %s"),
+					     srcpath, strerror(errno));
+				goto err_dir;
+			}
+			buf[len] = '\0';
+			if (symlink(buf, path) == -1) {
+				if (errno == EEXIST)
+					continue;
+				lu_error_new(error, lu_error_generic,
+					     _("Error creating `%s': %s"),
+					     path, strerror(errno));
+				goto err_dir;
+			}
+			if (lchown(path, owner, st.st_gid ?: group) == -1
+			    && errno != EPERM && errno != EOPNOTSUPP) {
+				lu_error_new(error, lu_error_generic,
+					     _("Error changing owner of `%s': "
+					       "%s"), dest, strerror(errno));
+				goto err_dir;
+			}
+			utime(path, &timebuf);
+			continue;
+		}
+
+		/* If it's a regular file, copy it. */
+		if (S_ISREG(st.st_mode)) {
+			off_t offset;
+
+			/* Open both the input and output files.  If we fail to
+			   do either, we have to give up. */
+			ifd = open(srcpath, O_RDONLY);
+			if (ifd == -1) {
+				lu_error_new(error, lu_error_open,
+					     _("Error reading `%s': %s"),
+					     srcpath, strerror(errno));
+				goto err_dir;
+			}
+			ofd = open(path, O_EXCL | O_CREAT | O_WRONLY,
+				   st.st_mode);
+			if (ofd == -1) {
+				if (errno == EEXIST) {
+					close(ifd);
+					continue;
 				}
-				/* Set the date on the directory. */
-				utime(path, &timebuf);
-				continue;
+				lu_error_new(error, lu_error_open,
+					     _("Error writing `%s': %s"),
+					     path, strerror(errno));
+				goto err_ifd;
 			}
 
-			/* If it's a symlink, duplicate it. */
-			if (S_ISLNK(st.st_mode)) {
-				ssize_t len;
+			/* Now just copy the data. */
+			for (;;) {
+				ssize_t left;
+				char *p;
 
-				len = readlink(skelpath, buf, sizeof(buf) - 1);
-				if (len == -1) {
-					lu_error_new(error, lu_error_generic,
-						     _("Error reading `%s': %s"),
-						     skelpath,
-						     strerror(errno));
-					goto err_dir;
-				}
-				buf[len] = '\0';
-				if (symlink(buf, path) == -1) {
-					if (errno == EEXIST)
+				left = read(ifd, &buf, sizeof(buf));
+				if (left == -1) {
+					if (errno == EINTR)
 						continue;
-					lu_error_new(error, lu_error_generic,
-						     _("Error creating `%s': %s"),
-						     path, strerror(errno));
-					goto err_dir;
-				}
-				if (lchown(path, owner, st.st_gid ?: group) == -1
-				    && errno != EPERM && errno != EOPNOTSUPP) {
-					lu_error_new(error, lu_error_generic,
-						     _("Error changing owner of `%s': %s"),
-						     directory,
+					lu_error_new(error, lu_error_read,
+						     _("Error reading `%s': "
+						       "%s"), srcpath,
 						     strerror(errno));
-					goto err_dir;
+					goto err_ofd;
 				}
-				utime(path, &timebuf);
-				continue;
-			}
+				if (left == 0)
+					break;
+				p = buf;
+				while (left > 0) {
+					ssize_t out;
 
-			/* If it's a regular file, copy it. */
-			if (S_ISREG(st.st_mode)) {
-				/* Open both the input and output
-				 * files.  If we fail to do either,
-				 * we have to give up. */
-				ifd = open(skelpath, O_RDONLY);
-				if (ifd == -1) {
-					lu_error_new(error, lu_error_open,
-						     _("Error reading `%s': %s"),
-						     skelpath,
-						     strerror(errno));
-					goto err_dir;
-				}
-				ofd = open(path, O_EXCL | O_CREAT | O_WRONLY,
-					   st.st_mode);
-				if (ofd == -1) {
-					if (errno == EEXIST) {
-						close(ofd);
-						continue;
-					}
-					lu_error_new(error, lu_error_open,
-						     _("Error writing `%s': %s"),
-						     path, strerror(errno));
-					goto err_ifd;
-				}
-
-				/* Now just copy the data. */
-				for (;;) {
-					ssize_t left;
-					char *p;
-
-					left = read(ifd, &buf, sizeof(buf));
-					if (left == -1) {
+					out = write(ofd, p, left);
+					if (out == -1) {
 						if (errno == EINTR)
 							continue;
 						lu_error_new(error,
-							     lu_error_read,
-							     _("Error reading `%s': %s"),
-							     skelpath,
-							     strerror(errno));
-						goto err_ofd;
-					}
-					if (left == 0)
-						break;
-					p = buf;
-					while (left > 0) {
-						ssize_t out;
-
-						out = write(ofd, p, left);
-						if (out == -1) {
-							if (errno == EINTR)
-								continue;
-							lu_error_new(error,
-								     lu_error_write,
-								     _("Error writing `%s': %s"),
-								     path,
-								     strerror(errno));
-							goto err_ofd;
-						}
-						p += out;
-						left -= out;
-					}
-				}
-
-				/* Close the files. */
-				offset = lseek(ofd, 0, SEEK_CUR);
-				if (offset != ((off_t) -1)) {
-					if (ftruncate(ofd, offset) == -1) {
-						lu_error_new(error,
-							     lu_error_generic,
-							     _("Error writing `%s': %s"),
+							     lu_error_write,
+							     _("Error writing "
+							       "`%s': %s"),
 							     path,
 							     strerror(errno));
 						goto err_ofd;
 					}
+					p += out;
+					left -= out;
 				}
-				close (ifd);
-				close (ofd);
-
-				/* Set the ownership and timestamp on
-				 * the new file. */
-				if (chown(path, owner, st.st_gid ?: group) == -1
-				    && errno != EPERM) {
-					lu_error_new(error, lu_error_generic,
-						     _("Error changing owner of `%s': %s"),
-						     directory,
-						     strerror(errno));
-					goto err_dir;
-				}
-				utime(path, &timebuf);
-				continue;
 			}
-			/* Note that we don't copy device specials. */
+
+			/* Close the files. */
+			offset = lseek(ofd, 0, SEEK_CUR);
+			if (offset != ((off_t) -1)) {
+				if (ftruncate(ofd, offset) == -1) {
+					lu_error_new(error, lu_error_generic,
+						     _("Error writing `%s': "
+						       "%s"), path,
+						     strerror(errno));
+					goto err_ofd;
+				}
+			}
+			close (ifd);
+			close (ofd);
+
+			/* Set the ownership and timestamp on the new file. */
+			if (chown(path, owner, st.st_gid ?: group) == -1
+			    && errno != EPERM) {
+				lu_error_new(error, lu_error_generic,
+					     _("Error changing owner of `%s': "
+					       "%s"), dest, strerror(errno));
+				goto err_dir;
+			}
+			utime(path, &timebuf);
+			continue;
 		}
+		/* Note that we don't copy device specials. */
 	}
 	ret = TRUE;
 	goto err_dir;
@@ -328,14 +316,26 @@ lu_homedir_populate(const char *skeleton, const char *directory,
 	return ret;
 }
 
+/* Populate a user's home directory, copying data from a named skeleton
+   directory (or default if skeleton is NULL), setting all ownerships as
+   given, and setting the mode of the top-level directory as given. */
+gboolean
+lu_homedir_populate(struct lu_context *ctx, const char *skeleton,
+		    const char *directory, uid_t owner, gid_t group,
+		    mode_t mode, struct lu_error **error)
+{
+	if (skeleton == NULL)
+		skeleton = lu_cfg_read_single(ctx, "defaults/skeleton",
+					      "/etc/skel");
+	return lu_homedir_copy(skeleton, directory, owner, group, mode, error);
+}
+
 /* Recursively remove a user's home (or really, any) directory. */
 gboolean
 lu_homedir_remove(const char *directory, struct lu_error ** error)
 {
 	struct dirent *ent;
 	DIR *dir;
-	struct stat st;
-	char path[PATH_MAX];
 
 	LU_ERROR_CHECK(error);
 
@@ -350,6 +350,9 @@ lu_homedir_remove(const char *directory, struct lu_error ** error)
 
 	/* Iterate over all of its contents. */
 	while ((ent = readdir(dir)) != NULL) {
+		struct stat st;
+		char path[PATH_MAX];
+
 		/* Skip over the self and parent hard links. */
 		if (strcmp(ent->d_name, ".") == 0) {
 			continue;
@@ -412,12 +415,10 @@ lu_homedir_move(const char *oldhome, const char *newhome,
 	/* If the directory exists... */
 	if (stat(oldhome, &st) != -1) {
 		/* ... and we can copy it ... */
-		if (lu_homedir_populate(oldhome, newhome,
-					st.st_uid, st.st_gid, st.st_mode,
-					error)) {
+		if (lu_homedir_copy(oldhome, newhome, st.st_uid, st.st_gid,
+				    st.st_mode, error))
 			/* ... remove the old one. */
 			return lu_homedir_remove(oldhome, error);
-		}
 	}
 
 	return FALSE;
@@ -428,10 +429,10 @@ lu_homedir_move(const char *oldhome, const char *newhome,
 static char *
 lu_strconcat(char *existing, const char *appendee)
 {
-	char *tmp;
 	if (existing == NULL) {
 		existing = g_strdup(appendee);
 	} else {
+		char *tmp;
 		tmp = g_strconcat(existing, appendee, NULL);
 		g_free(existing);
 		existing = tmp;
@@ -533,7 +534,7 @@ lu_authenticate_unprivileged(const char *user, const char *appname)
 {
 	pam_handle_t *pamh;
 	struct pam_conv conv;
-	const void *puser = user;
+	const void *puser;
 	int ret;
 
 #if 0
@@ -544,11 +545,11 @@ lu_authenticate_unprivileged(const char *user, const char *appname)
 		/* Great!  We can drop privileges. */
 		if (setegid(getgid()) == -1) {
 			fprintf(stderr, _("Failed to drop privileges.\n"));
-			exit(1);
+			goto err;
 		}
 		if (seteuid(getuid()) == -1) {
 			fprintf(stderr, _("Failed to drop privileges.\n"));
-			exit(1);
+			goto err;
 		}
 		return;
 	}
@@ -557,7 +558,7 @@ lu_authenticate_unprivileged(const char *user, const char *appname)
 	lu_get_prompter(ctx, &data.prompt, &data.callback_data);
 	if (data.prompt == NULL) {
 		fprintf(stderr, _("Internal error.\n"));
-		exit(1);
+		goto err;
 	}
 
 	conv.conv = lu_converse;
@@ -569,6 +570,7 @@ lu_authenticate_unprivileged(const char *user, const char *appname)
 
 #ifdef WITH_SELINUX
 	if (is_selinux_enabled() > 0) {
+		/* FIXME: PASSWD_CHSH, PASSWD_PASSWD ? */
 		if (getuid() == 0 && check_access(user, PASSWD__CHFN) != 0) {
 			security_context_t user_context;
 
@@ -581,14 +583,14 @@ lu_authenticate_unprivileged(const char *user, const char *appname)
 				: _("Unknown user context"), user);
 			if (user_context != NULL)
 				freecon(user_context);
-			exit(1);
+			goto err;
 		}
 		/* FIXME: is this right for lpasswd? */
 		if (setup_default_context("/etc/passwd") != 0) {
 			fprintf(stderr,
 				_("Can't set default context for "
 				  "/etc/passwd\n"));
-			exit(1);
+			goto err;
 		}
 	}
 #endif
@@ -596,17 +598,18 @@ lu_authenticate_unprivileged(const char *user, const char *appname)
 	/* Start up PAM. */
 	if (pam_start(appname, user, &conv, &pamh) != PAM_SUCCESS) {
 		fprintf(stderr, _("Error initializing PAM.\n"));
-		exit(1);
+		goto err;
 	}
 
 	/* Use PAM to authenticate the user. */
 	ret = pam_authenticate(pamh, 0);
 	if (ret != PAM_SUCCESS) {
-		pam_get_item(pamh, PAM_USER, &puser);
+		if (pam_get_item(pamh, PAM_USER, &puser) != PAM_SUCCESS
+		    || puser == NULL)
+			puser = user;
 		fprintf(stderr, _("Authentication failed for %s.\n"),
 			(const char *)puser);
-		pam_end(pamh, 0);
-		exit(1);
+		goto err_pam;
 	}
 
 	/* Make sure we authenticated the user we wanted to authenticate. */
@@ -614,37 +617,47 @@ lu_authenticate_unprivileged(const char *user, const char *appname)
 	if (ret != PAM_SUCCESS) {
 		fprintf(stderr, _("Internal PAM error `%s'.\n"),
 			pam_strerror(pamh, ret));
-		pam_end(pamh, 0);
-		exit(1);
+		goto err_pam;
+	}
+	if (puser == NULL) {
+		fprintf(stderr, _("Unknown user authenticated.\n"));
+		goto err_pam;
 	}
 	if (strcmp(puser, user) != 0) {
 		fprintf(stderr, _("User mismatch.\n"));
-		pam_end(pamh, 0);
-		exit(1);
+		goto err_pam;
 	}
 
 	/* Check if the user is allowed to run this program. */
-	if (pam_acct_mgmt(pamh, 0) != PAM_SUCCESS) {
-		puser = user;
-		pam_get_item(pamh, PAM_USER, &puser);
+	ret = pam_acct_mgmt(pamh, 0);
+	if (ret != PAM_SUCCESS) {
+		if (pam_get_item(pamh, PAM_USER, &puser) != PAM_SUCCESS
+		    || puser == NULL)
+			puser = user;
 		fprintf(stderr, _("Authentication failed for %s.\n"),
 			(const char *)puser);
-		pam_end(pamh, 0);
-		exit(1);
+		goto err_pam;
 	}
 
 	/* Clean up -- we're done. */
-	pam_end(pamh, 0);
+	pam_end(pamh, PAM_SUCCESS);
+	return;
+
+err_pam:
+	pam_end(pamh, ret);
+err:
+	exit(1);
 }
 
 /* Send nscd an arbitrary signal. */
-void
+static void
 lu_signal_nscd(int signum)
 {
 	FILE *fp;
-	char buf[LINE_MAX];
 	/* If it's running, then its PID is in this file.  Open it. */
 	if ((fp = fopen("/var/run/nscd.pid", "r")) != NULL) {
+		char buf[LINE_MAX];
+
 		/* Read the PID. */
 		memset(buf, 0, sizeof(buf));
 		/* If the PID is sane, send it a signal. */
@@ -676,11 +689,8 @@ lu_mailspool_create_remove(struct lu_context *ctx, struct lu_ent *ent,
 	uid_t uid;
 	gid_t gid;
 	char *p, *username;
-	struct group grp, *err;
 	struct lu_ent *groupEnt;
 	struct lu_error *error = NULL;
-	char buf[LINE_MAX * 4];
-	int fd;
 
 	/* Find the GID of the owner of the file. */
 	gid = LU_VALUE_INVALID_ID;
@@ -692,10 +702,15 @@ lu_mailspool_create_remove(struct lu_context *ctx, struct lu_ent *ent,
 			gid = lu_value_get_id(value);
 		}
 	}
+	if (error != NULL)
+		lu_error_free(&error);
 	lu_ent_free(groupEnt);
 
 	/* Er, okay.  Check with libc. */
 	if (gid == LU_VALUE_INVALID_ID) {
+		struct group grp, *err;
+		char buf[LINE_MAX * 4];
+
 		if ((getgrnam_r("mail", &grp, buf, sizeof(buf), &err) == 0) &&
 		    (err == &grp)) {
 			gid = grp.gr_gid;
@@ -738,6 +753,8 @@ lu_mailspool_create_remove(struct lu_context *ctx, struct lu_ent *ent,
 	p = g_strdup_printf("%s/%s", spooldir, username);
 	g_free(username);
 	if (action) {
+		int fd;
+
 		fd = open(p, O_WRONLY | O_CREAT, 0);
 		if (fd != -1) {
 			gboolean res = TRUE;
