@@ -1,4 +1,4 @@
-/* Copyright (C) 2000-2002, 2004 Red Hat, Inc.
+/* Copyright (C) 2000-2002, 2004, 2005, 2006 Red Hat, Inc.
  *
  * This is free software; you can redistribute it and/or modify it under
  * the terms of the GNU Library General Public License as published by
@@ -15,7 +15,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#ident "$Id: user.c,v 1.78 2005/09/12 23:24:37 mitr Exp $"
+#ident "$Id: user.c,v 1.81 2006/03/06 03:02:52 mitr Exp $"
 
 #ifdef HAVE_CONFIG_H
 #include "../config.h"
@@ -79,7 +79,7 @@ lu_start(const char *auth_name, enum lu_entity_type auth_type,
 	 lu_prompt_fn *prompter, gpointer prompter_data,
 	 struct lu_error **error)
 {
-	struct lu_context *ctx = NULL;
+	struct lu_context *ctx;
 
 	LU_ERROR_CHECK(error);
 
@@ -92,16 +92,13 @@ lu_start(const char *auth_name, enum lu_entity_type auth_type,
 	/* Allocate space for the context. */
 	ctx = g_malloc0(sizeof(struct lu_context));
 
-	/* Create a configuration structure. */
-	if (lu_cfg_init(ctx, error) == FALSE) {
-		/* If there's an error, lu_cfg_init() sets it. */
-		g_free(ctx);
-		return NULL;
-	}
-
-	/* Initialize the rest of the fields. */
 	ctx->scache = lu_string_cache_new(TRUE);
 
+	/* Create a configuration structure. */
+	if (lu_cfg_init(ctx, error) == FALSE)
+		/* If there's an error, lu_cfg_init() sets it. */
+		goto err_scache;
+
 	ctx->auth_name = ctx->scache->cache(ctx->scache, auth_name);
 	ctx->auth_type = auth_type;
 
@@ -124,19 +121,23 @@ lu_start(const char *auth_name, enum lu_entity_type auth_type,
 	}
 
 	/* Load the modules. */
-	if (!lu_modules_load(ctx, modules, &ctx->module_names, error)) {
-		/* lu_module_load sets errors */
-		g_free(ctx);
-		return NULL;
-	}
+	if (!lu_modules_load(ctx, modules, &ctx->module_names, error))
+		goto err_modules; /* lu_module_load sets errors */
 	if (!lu_modules_load(ctx, create_modules, &ctx->create_module_names,
-			     error)) {
-		/* lu_module_load sets errors */
-		g_free(ctx);
-		return NULL;
-	}
+			     error))
+		goto err_module_names; /* lu_module_load sets errors */
 
 	return ctx;
+
+err_module_names:
+	g_value_array_free(ctx->module_names);
+	g_tree_foreach(ctx->modules, lu_module_unload, NULL);
+err_modules:
+	g_tree_destroy(ctx->modules);
+err_scache:
+	ctx->scache->free(ctx->scache);
+	g_free(ctx);
+	return NULL;
 }
 
 void
@@ -144,19 +145,15 @@ lu_end(struct lu_context *context)
 {
 	g_assert(context != NULL);
 
-	if (context->modules != NULL) {
-		g_tree_foreach(context->modules, lu_module_unload, NULL);
-		g_tree_destroy(context->modules);
-	}
+	g_tree_foreach(context->modules, lu_module_unload, NULL);
+	g_tree_destroy(context->modules);
 
 	g_value_array_free(context->create_module_names);
 	g_value_array_free(context->module_names);
 
 	lu_cfg_done(context);
 
-	if (context->scache != NULL) {
-		context->scache->free(context->scache);
-	}
+	context->scache->free(context->scache);
 
 	memset(context, 0, sizeof(struct lu_context));
 
@@ -175,8 +172,9 @@ extract_name(struct lu_ent *ent)
 	if (array == NULL) {
 		array = lu_ent_get_current(ent,
 					   ent->type == lu_user ? LU_USERNAME : LU_GROUPNAME);
+		if (array == NULL)
+			return NULL;
 	}
-	g_return_val_if_fail(array != NULL, NULL);
 	value = g_value_array_get_nth(array, 0);
 	g_return_val_if_fail(value != NULL, NULL);
 	return ent->cache->cache(ent->cache, g_value_get_string(value));
@@ -269,51 +267,88 @@ extract_id(struct lu_ent *ent)
 	if (array == NULL) {
 		array = lu_ent_get_current(ent,
 					   ent->type == lu_user ? LU_UIDNUMBER : LU_GIDNUMBER);
+		if (array == NULL)
+			return LU_VALUE_INVALID_ID;
 	}
-	g_return_val_if_fail(array != NULL, LU_VALUE_INVALID_ID);
 	value = g_value_array_get_nth(array, 0);
 	g_return_val_if_fail(value != NULL, LU_VALUE_INVALID_ID);
 	return lu_value_get_id(value);
 }
 
 static uid_t
-convert_user_name_to_id(struct lu_context *context, const char *sdata)
+convert_user_name_to_id(struct lu_context *context, const char *sdata,
+			struct lu_error **error)
 {
 	struct lu_ent *ent;
 	uid_t ret = LU_VALUE_INVALID_ID;
 	char buf[LINE_MAX * 4];
 	struct passwd *err, passwd;
-	struct lu_error *error = NULL;
+
 	if ((getpwnam_r(sdata, &passwd, buf, sizeof(buf), &err) == 0) &&
 	    (err == &passwd))
 		return passwd.pw_uid;
 	ent = lu_ent_new();
-	if (lu_user_lookup_name(context, sdata, ent, &error) == TRUE) {
+	if (lu_user_lookup_name(context, sdata, ent, error) == TRUE) {
 		ret = extract_id(ent);
+		if (ret == LU_VALUE_INVALID_ID)
+			lu_error_new(error, lu_error_generic,
+				     _("user %s has no UID"), sdata);
 	}
 	lu_ent_free(ent);
 	return ret;
 }
 
 static gid_t
-convert_group_name_to_id(struct lu_context *context, const char *sdata)
+convert_group_name_to_id(struct lu_context *context, const char *sdata,
+			 struct lu_error **error)
 {
 	struct lu_ent *ent;
 	gid_t ret = LU_VALUE_INVALID_ID;
 	char buf[LINE_MAX * 4];
 	struct group *err, group;
-	struct lu_error *error = NULL;
+
 	if ((getgrnam_r(sdata, &group, buf, sizeof(buf), &err) == 0) &&
 	    (err == &group))
 		return group.gr_gid;
 	ent = lu_ent_new();
-	if (lu_group_lookup_name(context, sdata, ent, &error) == TRUE) {
+	if (lu_group_lookup_name(context, sdata, ent, error) == TRUE) {
 		ret = extract_id(ent);
+		if (ret == LU_VALUE_INVALID_ID)
+			lu_error_new(error, lu_error_generic,
+				     _("group %s has no GID"), sdata);
 	}
 	lu_ent_free(ent);
 	return ret;
 }
 
+static gboolean ent_has_name_and_id(struct lu_ent *ent,
+				    struct lu_error **error)
+{
+	const char *name;
+	id_t id;
+
+	g_return_val_if_fail(ent->type == lu_user || ent->type == lu_group,
+			     FALSE);
+	name = extract_name(ent);
+	id = extract_id(ent);
+	if (name != NULL && id != LU_VALUE_INVALID_ID)
+		return TRUE;
+	if (id != LU_VALUE_INVALID_ID)
+		lu_error_new(error, lu_error_generic,
+			     ent->type == lu_user ? _("user %jd has no name")
+			     : _("group %jd has no name"),
+			     (intmax_t)id);
+	else if (name != NULL)
+		lu_error_new(error, lu_error_generic, ent->type == lu_user
+			     ? _("user %s has no UID")
+			     : _("group %s has no GID"), name);
+	else
+		lu_error_new(error, lu_error_generic, ent->type == lu_user
+			     ? _("user has neither a name nor an UID")
+			     : _("group has neither a name nor a GID"));
+	return FALSE;
+}
+
 static gboolean lu_refresh_int(struct lu_context *context,
 			       struct lu_ent *entity,
 			       struct lu_error **error);
@@ -583,12 +618,16 @@ logic_or(gboolean a, gboolean b)
 static void
 remove_duplicate_values(GValueArray *array)
 {
-	size_t i, j;
-	GValue *ivalue, *jvalue;
+	size_t i;
 
 	for (i = 0; i < array->n_values; i++) {
+		size_t j;
+		GValue *ivalue;
+
 		ivalue = g_value_array_get_nth(array, i);
 		for (j = i + 1; j < array->n_values; j++) {
+			GValue *jvalue;
+
 			jvalue = g_value_array_get_nth(array, j);
 			if (G_VALUE_TYPE(ivalue) == G_VALUE_TYPE(jvalue)
 			    && lu_values_equal(ivalue, jvalue)) {
@@ -609,23 +648,22 @@ compare_strings(gconstpointer a, gconstpointer b, gpointer data)
 static GPtrArray *
 merge_ent_array_duplicates(GPtrArray *array)
 {
-	GPtrArray *ret = NULL;
-	size_t i, j;
-	const char *attr;
-	struct lu_ent *current, *saved;
-	GValueArray *values;
-	GValue *value;
-	GList *attributes, *list;
-	GTree *users, *groups, *tree;
+	GPtrArray *ret;
+	size_t i;
+	GTree *users, *groups;
+
 	g_return_val_if_fail(array != NULL, NULL);
-	/* We need four trees to hold the known entities. */
 	users = g_tree_new_full(compare_strings, NULL, g_free, NULL);
 	groups = g_tree_new_full(compare_strings, NULL, g_free, NULL);
 	/* A structure to hold the new list. */
 	ret = g_ptr_array_new();
 	/* Iterate over every entity in the incoming list. */
 	for (i = 0; i < array->len; i++) {
+		struct lu_ent *current, *saved;
 		char *key;
+		GValueArray *values;
+		GValue *value;
+		GTree *tree;
 
 		current = g_ptr_array_index(array, i);
 		key = NULL;
@@ -644,7 +682,6 @@ merge_ent_array_duplicates(GPtrArray *array)
 			g_assert_not_reached();
 		}
 		value = g_value_array_get_nth(values, 0);
-		/* Convert that name or number to a quark. */
 		key = lu_value_strdup(value);
 		/* Check if there's already an entity with that name. */
 		saved = g_tree_lookup(tree, key);
@@ -653,6 +690,10 @@ merge_ent_array_duplicates(GPtrArray *array)
 			g_tree_insert(tree, key, current);
 			g_ptr_array_add(ret, current);
 		} else {
+			GList *attributes, *list;
+			const char *attr;
+			size_t j;
+
 			g_free (key);
 			/* Merge all of its data into the existing one; first,
 			 * the current data. */
@@ -671,6 +712,7 @@ merge_ent_array_duplicates(GPtrArray *array)
 			g_list_free(list);
 			/* Merge the pending data. */
 			attributes = lu_ent_get_attributes(current);
+			list = attributes;
 			while (attributes != NULL) {
 				attr = (const char *)attributes->data;
 				values = lu_ent_get(current, attr);
@@ -708,16 +750,9 @@ run_list(struct lu_context *context,
 	 gpointer ret,
 	 struct lu_error **firsterror)
 {
-	struct lu_module *module;
-	GPtrArray *ptr_array = NULL, *tmp_ptr_array = NULL;
-	GValueArray *value_array = NULL, *tmp_value_array = NULL;
-	GValue *value;
-	gpointer scratch;
-	struct lu_ent *tmp_ent;
-	char *name;
-	gboolean success, tsuccess;
+	gboolean success;
 	struct lu_error *lasterror = NULL;
-	size_t i, j;
+	size_t i;
 
 	LU_ERROR_CHECK(firsterror);
 
@@ -764,6 +799,12 @@ run_list(struct lu_context *context,
 
 	success = FALSE;
 	for (i = 0; i < list->n_values; i++) {
+		struct lu_module *module;
+		gpointer scratch;
+		GValue *value;
+		char *name;
+		gboolean tsuccess;
+
 		value = g_value_array_get_nth(list, i);
 		name = g_value_dup_string(value);
 		module = g_tree_lookup(context->modules, name);
@@ -774,6 +815,10 @@ run_list(struct lu_context *context,
 				      sdata, ldata, entity, &scratch,
 				      &lasterror);
 		if (scratch != NULL) switch (id) {
+			GPtrArray *ptr_array, *tmp_ptr_array;
+			GValueArray *value_array, *tmp_value_array;
+			size_t j;
+
 			case users_enumerate:
 			case users_enumerate_by_group:
 			case groups_enumerate:
@@ -807,6 +852,8 @@ run_list(struct lu_context *context,
 				}
 				if (tmp_ptr_array != NULL) {
 					for (j = 0; j < tmp_ptr_array->len; j++) {
+						struct lu_ent *tmp_ent;
+
 						tmp_ent = g_ptr_array_index(tmp_ptr_array,
 									    j);
 						g_ptr_array_add(ptr_array, tmp_ent);
@@ -869,7 +916,6 @@ lu_refresh_int(struct lu_context *context, struct lu_ent *entity,
 {
 	enum lu_dispatch_id id = 0;
 	const char *sdata;
-	id_t ldata;
 	gpointer scratch = NULL;
 	g_return_val_if_fail((entity->type == lu_user) ||
 			     (entity->type == lu_group),
@@ -883,9 +929,10 @@ lu_refresh_int(struct lu_context *context, struct lu_ent *entity,
 		g_assert_not_reached();
 	}
 	sdata = extract_name(entity);
-	ldata = extract_id(entity);
+	if (sdata == NULL)
+		return FALSE;
 	if (run_list(context, entity->modules, logic_and, id,
-		     sdata, ldata, entity, &scratch, error)) {
+		     sdata, LU_VALUE_INVALID_ID, entity, &scratch, error)) {
 		lu_ent_revert(entity);
 		return TRUE;
 	}
@@ -900,13 +947,11 @@ lu_dispatch(struct lu_context *context,
 	    gpointer ret,
 	    struct lu_error **error)
 {
-	struct lu_ent *tmp = NULL;
+	struct lu_ent *tmp;
 	gboolean success;
 	GValueArray *values = NULL;
 	GPtrArray *ptrs = NULL;
-	GValue *value = NULL;
 	gpointer scratch = NULL;
-	size_t i;
 
 	LU_ERROR_CHECK(error);
 
@@ -942,6 +987,8 @@ lu_dispatch(struct lu_context *context,
 			}
 			values = lu_ent_get_current(tmp, attr);
 			if (values != NULL) {
+				GValue *value;
+
 				value = g_value_array_get_nth(values, 0);
 				attr = g_value_get_string(value);
 				sdata = tmp->cache->cache(tmp->cache, attr);
@@ -958,11 +1005,10 @@ lu_dispatch(struct lu_context *context,
 	case group_lookup_name:
 		/* Make sure data items are right for this call. */
 		g_assert(sdata != NULL);
-		ldata = LU_VALUE_INVALID_ID;
 		/* Run the list. */
-		if (run_list(context, context->module_names,
-			    logic_or, id,
-			    sdata, ldata, tmp, &scratch, error)) {
+		if (run_list(context, context->module_names, logic_or, id,
+			     sdata, LU_VALUE_INVALID_ID, tmp, &scratch,
+			     error)) {
 			if (entity != NULL) {
 				lu_ent_revert(tmp);
 				lu_ent_copy(tmp, entity);
@@ -987,15 +1033,12 @@ lu_dispatch(struct lu_context *context,
 	case user_add_prep:
 	case group_add_prep:
 		/* Make sure we have both name and ID here. */
-		sdata = sdata ?: extract_name(tmp);
-		if (ldata == LU_VALUE_INVALID_ID)
-			ldata = extract_id(tmp);
-		g_return_val_if_fail(sdata != NULL, FALSE);
-		g_return_val_if_fail(ldata != LU_VALUE_INVALID_ID, FALSE);
+		if (ent_has_name_and_id(tmp, error) == FALSE)
+			break;
 		/* Run the checks and preps. */
-		if (run_list(context, context->create_module_names,
-			    logic_and, id,
-			    sdata, ldata, tmp, &scratch, error)) {
+		if (run_list(context, context->create_module_names, logic_and,
+			     id, NULL, LU_VALUE_INVALID_ID, tmp, &scratch,
+			     error)) {
 			if (entity != NULL) {
 				lu_ent_copy(tmp, entity);
 			}
@@ -1005,15 +1048,12 @@ lu_dispatch(struct lu_context *context,
 	case user_add:
 	case group_add:
 		/* Make sure we have both name and ID here. */
-		sdata = sdata ?: extract_name(tmp);
-		if (ldata == LU_VALUE_INVALID_ID)
-			ldata = extract_id(tmp);
-		g_return_val_if_fail(sdata != NULL, FALSE);
-		g_return_val_if_fail(ldata != LU_VALUE_INVALID_ID, FALSE);
+		if (ent_has_name_and_id(tmp, error) == FALSE)
+			break;
 		/* Add the account. */
 		if (run_list(context, context->create_module_names,
-			    logic_and, id,
-			    sdata, ldata, tmp, &scratch, error)) {
+			     logic_and, id, NULL, LU_VALUE_INVALID_ID,
+			     tmp, &scratch, error)) {
 			if (entity != NULL) {
 				lu_ent_copy(tmp, entity);
 			}
@@ -1023,18 +1063,13 @@ lu_dispatch(struct lu_context *context,
 	case user_mod:
 	case group_mod:
 		/* Make sure we have both name and ID here. */
-		/* FIXME: sdata, ldata contain new values (and are not even
-		   used). */
-		sdata = sdata ?: extract_name(tmp);
-		if (ldata == LU_VALUE_INVALID_ID)
-			ldata = extract_id(tmp);
-		g_return_val_if_fail(sdata != NULL, FALSE);
-		g_return_val_if_fail(ldata != LU_VALUE_INVALID_ID, FALSE);
+		/* FIXME: this checks current, not pending values */
+		if (ent_has_name_and_id(tmp, error) == FALSE)
+			break;
 		/* Make the changes. */
 		g_assert(entity != NULL);
-		if (run_list(context, entity->modules,
-			    logic_and, id,
-			    sdata, ldata, tmp, &scratch, error)) {
+		if (run_list(context, entity->modules, logic_and, id, NULL,
+			     LU_VALUE_INVALID_ID, tmp, &scratch, error)) {
 			lu_ent_commit(tmp);
 			lu_ent_copy(tmp, entity);
 			success = TRUE;
@@ -1049,16 +1084,12 @@ lu_dispatch(struct lu_context *context,
 	case group_unlock:
 	case group_unlock_nonempty:
 		/* Make sure we have both name and ID here. */
-		sdata = sdata ?: extract_name(tmp);
-		if (ldata == LU_VALUE_INVALID_ID)
-			ldata = extract_id(tmp);
-		g_return_val_if_fail(sdata != NULL, FALSE);
-		g_return_val_if_fail(ldata != LU_VALUE_INVALID_ID, FALSE);
+		if (ent_has_name_and_id(tmp, error) == FALSE)
+			break;
 		/* Make the changes. */
 		g_assert(entity != NULL);
-		if (run_list(context, entity->modules,
-			    logic_and, id,
-			    sdata, ldata, tmp, &scratch, error)) {
+		if (run_list(context, entity->modules, logic_and, id, NULL,
+			     LU_VALUE_INVALID_ID, tmp, &scratch, error)) {
 			lu_ent_revert(tmp);
 			lu_ent_copy(tmp, entity);
 			success = TRUE;
@@ -1073,9 +1104,8 @@ lu_dispatch(struct lu_context *context,
 	case group_removepass:
 		/* Make the changes. */
 		g_assert(entity != NULL);
-		if (run_list(context, entity->modules,
-			    logic_and, id,
-			    sdata, ldata, tmp, &scratch, error)) {
+		if (run_list(context, entity->modules, logic_and, id, sdata,
+			     LU_VALUE_INVALID_ID, tmp, &scratch, error)) {
 			lu_ent_revert(tmp);
 			lu_ent_copy(tmp, entity);
 			success = TRUE;
@@ -1084,16 +1114,12 @@ lu_dispatch(struct lu_context *context,
 	case user_is_locked:
 	case group_is_locked:
 		/* Make sure we have both name and ID here. */
-		sdata = sdata ?: extract_name(tmp);
-		if (ldata == LU_VALUE_INVALID_ID)
-			ldata = extract_id(tmp);
-		g_return_val_if_fail(sdata != NULL, FALSE);
-		g_return_val_if_fail(ldata != LU_VALUE_INVALID_ID, FALSE);
+		if (ent_has_name_and_id(tmp, error) == FALSE)
+			break;
 		/* Run the checks. */
 		g_assert(entity != NULL);
-		if (run_list(context, entity->modules,
-			    logic_or, id,
-			    sdata, ldata, tmp, &scratch, error)) {
+		if (run_list(context, entity->modules, logic_or, id, NULL,
+			     LU_VALUE_INVALID_ID, tmp, &scratch, error)) {
 			lu_ent_copy(tmp, entity);
 			success = TRUE;
 		}
@@ -1102,15 +1128,15 @@ lu_dispatch(struct lu_context *context,
 	case groups_enumerate_by_user:
 		/* Make sure we have both name and ID here. */
 		g_return_val_if_fail(sdata != NULL, FALSE);
-		if (id == users_enumerate_by_group) {
-			ldata = convert_group_name_to_id(context, sdata);
-		} else
-		if (id == groups_enumerate_by_user) {
-			ldata = convert_user_name_to_id(context, sdata);
-		} else {
+		if (id == users_enumerate_by_group)
+			ldata = convert_group_name_to_id(context, sdata,
+							 error);
+		else if (id == groups_enumerate_by_user)
+			ldata = convert_user_name_to_id(context, sdata, error);
+		else
 			g_assert_not_reached();
-		}
-		g_return_val_if_fail(ldata != LU_VALUE_INVALID_ID, FALSE);
+		if (ldata == LU_VALUE_INVALID_ID)
+			break;
 		/* fall through */
 	case users_enumerate:
 	case groups_enumerate:
@@ -1126,15 +1152,15 @@ lu_dispatch(struct lu_context *context,
 	case groups_enumerate_by_user_full:
 		/* Make sure we have both name and ID here. */
 		g_return_val_if_fail(sdata != NULL, FALSE);
-		if (id == users_enumerate_by_group_full) {
-			ldata = convert_group_name_to_id(context, sdata);
-		} else
-		if (id == groups_enumerate_by_user_full) {
-			ldata = convert_user_name_to_id(context, sdata);
-		} else {
+		if (id == users_enumerate_by_group)
+			ldata = convert_group_name_to_id(context, sdata,
+							 error);
+		else if (id == groups_enumerate_by_user)
+			ldata = convert_user_name_to_id(context, sdata, error);
+		else
 			g_assert_not_reached();
-		}
-		g_return_val_if_fail(ldata != LU_VALUE_INVALID_ID, FALSE);
+		if (ldata == LU_VALUE_INVALID_ID)
+			break;
 		/* fall through */
 	case users_enumerate_full:
 	case groups_enumerate_full:
@@ -1143,6 +1169,8 @@ lu_dispatch(struct lu_context *context,
 			    logic_or, id,
 			    sdata, ldata, tmp, &ptrs, error)) {
 			if (ptrs != NULL) {
+				size_t i;
+
 				for (i = 0; i < ptrs->len; i++) {
 					struct lu_ent *ent;
 					ent = g_ptr_array_index(ptrs, i);
@@ -1166,7 +1194,7 @@ lu_dispatch(struct lu_context *context,
 		}
 		break;
 	default:
-		g_assert(0);	/* not reached */
+		g_assert_not_reached();
 		break;
 	}
 	lu_ent_free(tmp);
@@ -1665,9 +1693,10 @@ lu_get_first_unused_id(struct lu_context *ctx,
 
 	ent = lu_ent_new();
 	if (type == lu_user) {
-		struct passwd pwd, *err;
 		struct lu_error *error = NULL;
 		do {
+			struct passwd pwd, *err;
+
 			/* There may be read-only sources of user information
 			 * on the system, and we want to avoid allocating an ID
 			 * that's already in use by a service we can't write
@@ -1690,9 +1719,10 @@ lu_get_first_unused_id(struct lu_context *ctx,
 			break;
 		} while (id != (id_t)-1);
 	} else if (type == lu_group) {
-		struct group grp, *err;
 		struct lu_error *error = NULL;
 		do {
+			struct group grp, *err;
+
 			/* There may be read-only sources of user information
 			 * on the system, and we want to avoid allocating an ID
 			 * that's already in use by a service we can't write
@@ -1726,9 +1756,8 @@ lu_default_int(struct lu_context *context, const char *name,
 {
 	GList *keys, *p;
 	GValue value;
-	char *cfgkey, *tmp, *end;
-	char buf[LINE_MAX * 4];
-	const char *top, *idkey, *idkeystring, *val, *key;
+	char *cfgkey;
+	const char *top, *idkey, *idkeystring, *val;
 	id_t id = DEFAULT_ID;
 	struct group grp, *err;
 	struct lu_error *error = NULL;
@@ -1752,6 +1781,8 @@ lu_default_int(struct lu_context *context, const char *name,
 	g_value_init(&value, G_TYPE_STRING);
 	g_value_set_string(&value, name);
 	if (ent->type == lu_user) {
+		char buf[LINE_MAX * 4];
+
 		lu_ent_clear(ent, LU_USERNAME);
 		lu_ent_add(ent, LU_USERNAME, &value);
 		/* Additionally, pick a default default group. */
@@ -1799,10 +1830,11 @@ lu_default_int(struct lu_context *context, const char *name,
 		}
 		if (val != NULL) {
 			intmax_t imax;
+			char *end;
 
 			errno = 0;
-			imax = strtoimax(val, &tmp, 10);
-			if (errno == 0 && *tmp == 0 && tmp != val
+			imax = strtoimax(val, &end, 10);
+			if (errno == 0 && *end == 0 && end != val
 			    && (id_t)imax == imax)
 				id = imax;
 			else
@@ -1821,9 +1853,7 @@ lu_default_int(struct lu_context *context, const char *name,
 	/* Now iterate to find the rest. */
 	keys = lu_cfg_read_keys(context, top);
 	for (p = keys; p && p->data; p = g_list_next(p)) {
-		intmax_t imax;
-
-		struct {
+		static const struct {
 			const char *realkey, *configkey;
 		} keymap[] = {
 			{LU_USERNAME, G_STRINGIFY_ARG(LU_USERNAME)},
@@ -1860,6 +1890,10 @@ lu_default_int(struct lu_context *context, const char *name,
 			{LU_HOMEPHONE, G_STRINGIFY_ARG(LU_HOMEPHONE)},
 			{LU_EMAIL, G_STRINGIFY_ARG(LU_EMAIL)},
 		};
+
+		intmax_t imax;
+		char *end, *tmp;
+		const char *key;
 		struct {
 			const char *format;
 			const char *value;
@@ -1871,7 +1905,7 @@ lu_default_int(struct lu_context *context, const char *name,
 		};
 
 		/* Possibly map the key to an internal name. */
-		key = (const char *) p->data;
+		key = p->data;
 		for (i = 0; i < G_N_ELEMENTS(keymap); i++) {
 			if (strcmp(key, keymap[i].configkey) == 0) {
 				key = keymap[i].realkey;
