@@ -1,4 +1,4 @@
-/* Copyright (C) 2001, 2002, 2004 Red Hat, Inc.
+/* Copyright (C) 2001, 2002, 2004, 2006 Red Hat, Inc.
  *
  * This is free software; you can redistribute it and/or modify it under
  * the terms of the GNU Library General Public License as published by
@@ -15,7 +15,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#ident "$Id: admin.c,v 1.40 2004/11/14 22:13:36 mitr Exp $"
+#ident "$Id: admin.c,v 1.43 2006/05/01 04:51:21 mitr Exp $"
 
 #include <Python.h>
 #ifdef HAVE_CONFIG_H
@@ -170,6 +170,8 @@ libuser_admin_lookup_user_name(PyObject *self, PyObject *args,
 		return libuser_wrap_ent(ent);
 	} else {
 		/* No such user.  Clean up and bug out. */
+		if (error)
+			lu_error_free(&error);
 		lu_ent_free(ent);
 		DEBUG_EXIT;
 		return Py_BuildValue("");
@@ -205,6 +207,8 @@ libuser_admin_lookup_user_id(PyObject *self, PyObject *args,
 		return libuser_wrap_ent(ent);
 	} else {
 		/* No such user.  Clean up and bug out. */
+		if (error != NULL)
+			lu_error_free(&error);
 		lu_ent_free(ent);
 		DEBUG_EXIT;
 		return Py_BuildValue("");
@@ -236,6 +240,8 @@ libuser_admin_lookup_group_name(PyObject *self, PyObject *args,
 		return libuser_wrap_ent(ent);
 	} else {
 		/* We've got nothing.  Return nothing. */
+		if (error != NULL)
+			lu_error_free(&error);
 		lu_ent_free(ent);
 		DEBUG_EXIT;
 		return Py_BuildValue("");
@@ -272,6 +278,8 @@ libuser_admin_lookup_group_id(PyObject *self, PyObject *args,
 		return libuser_wrap_ent(ent);
 	} else {
 		/* Clean up and exit, we have nothing to return. */
+		if (error != NULL)
+			lu_error_free(&error);
 		lu_ent_free(ent);
 		DEBUG_EXIT;
 		return Py_BuildValue("");
@@ -391,6 +399,7 @@ libuser_admin_wrap_boolean(PyObject *self, PyObject *args, PyObject *kwargs,
 	struct lu_error *error = NULL;
 	char *keywords[] = { "entity", NULL };
 	struct libuser_admin *me = (struct libuser_admin *) self;
+	gboolean ret;
 
 	DEBUG_ENTRY;
 	/* Expect a Python Entity object and maybe some other stuff we
@@ -400,8 +409,11 @@ libuser_admin_wrap_boolean(PyObject *self, PyObject *args, PyObject *kwargs,
 		DEBUG_EXIT;
 	}
 	/* Run the function. */
+	ret = fn(me->ctx, ent->ent, &error);
+	if (error != NULL)
+		lu_error_free(&error);
 	DEBUG_EXIT;
-	return Py_BuildValue("i", (fn(me->ctx, ent->ent, &error)) ? 1 : 0);
+	return Py_BuildValue("i", ret ? 1 : 0);
 }
 
 /* Wrap the setpass function for either type of entity. */
@@ -450,7 +462,8 @@ libuser_admin_create_home(PyObject *self, PyObject *args,
 			  PyObject *kwargs)
 {
 	struct libuser_entity *ent = NULL;
-	const char *dir = NULL, *skeleton = "/etc/skel";
+	struct lu_context *context;
+	const char *dir, *skeleton = NULL;
 	GValueArray *values;
 	GValue *value;
 	char *keywords[] = { "home", "skeleton", NULL };
@@ -461,6 +474,8 @@ libuser_admin_create_home(PyObject *self, PyObject *args,
 	(void)self;
 	DEBUG_ENTRY;
 
+	context = ((struct libuser_admin *)self)->ctx;
+
 	/* Expect an object and a string. */
 	if (!PyArg_ParseTupleAndKeywords(args, kwargs, "O!|s", keywords,
 					 &EntityType, &ent, &skeleton)) {
@@ -502,7 +517,7 @@ libuser_admin_create_home(PyObject *self, PyObject *args,
 	gidNumber = lu_value_get_id(value);
 
 	/* Attempt to populate the directory. */
-	if (lu_homedir_populate(skeleton, dir, uidNumber, gidNumber,
+	if (lu_homedir_populate(context, skeleton, dir, uidNumber, gidNumber,
 				0700, &error)) {
 		/* Success -- return an empty tuple. */
 		DEBUG_EXIT;
@@ -527,7 +542,7 @@ libuser_admin_remove_home(PyObject *self, PyObject *args,
 			  PyObject *kwargs)
 {
 	struct libuser_entity *ent = NULL;
-	const char *dir = NULL;
+	const char *dir;
 	GValueArray *values;
 	GValue *value;
 	char *keywords[] = { "home", NULL };
@@ -702,9 +717,12 @@ libuser_admin_add_user(PyObject *self, PyObject *args, PyObject *kwargs)
 	PyObject *ret = NULL;
 	PyObject *mkhomedir = self;
 	PyObject *mkmailspool = self;
+	PyObject *skeleton = NULL;
 	struct libuser_entity *ent = NULL;
 	struct lu_context *context = NULL;
-	char *keywords[] = { "entity", "mkhomedir", "mkmailspool", NULL };
+	char *keywords[] = {
+		"entity", "mkhomedir", "mkmailspool", "skeleton", NULL
+	};
 
 	DEBUG_ENTRY;
 
@@ -712,9 +730,10 @@ libuser_admin_add_user(PyObject *self, PyObject *args, PyObject *kwargs)
 
 	/* Expect an entity and a flag to tell us if we need to create the
 	 * user's home directory. */
-	if (!PyArg_ParseTupleAndKeywords(args, kwargs, "O!|OO", keywords,
+	if (!PyArg_ParseTupleAndKeywords(args, kwargs, "O!|OOO", keywords,
 					 &EntityType, &ent,
-					 &mkhomedir, &mkmailspool)) {
+					 &mkhomedir, &mkmailspool,
+					 &skeleton)) {
 		DEBUG_EXIT;
 		return NULL;
 	}
@@ -736,8 +755,13 @@ libuser_admin_add_user(PyObject *self, PyObject *args, PyObject *kwargs)
 			subargs = PyTuple_New(1);
 			Py_INCREF(ent);
 			PyTuple_SetItem(subargs, 0, (PyObject*) ent);
-			/* Create an empty dictionary for keyword args. */
+			/* Create a dictionary for keyword args. */
 			subkwargs = PyDict_New();
+			if (skeleton != NULL) {
+				Py_INCREF(skeleton);
+				PyDict_SetItemString(subkwargs, "skeleton",
+						     skeleton);
+			}
 			/* We'll return the result of the creation call. */
 			ret = libuser_admin_create_home(self, subargs,
 							subkwargs);
@@ -777,7 +801,7 @@ libuser_admin_modify_user(PyObject *self, PyObject *args,
 			  PyObject *kwargs)
 {
 	PyObject *ent = NULL;
-	PyObject *ret = NULL;
+	PyObject *ret;
 	PyObject *mvhomedir = NULL;
 	struct lu_ent *copy = NULL;
 	char *keywords[] = { "entity", "mvhomedir", NULL };
@@ -834,7 +858,7 @@ libuser_admin_delete_user(PyObject *self, PyObject *args,
 			  PyObject *kwargs)
 {
 	PyObject *ent = NULL;
-	PyObject *ret = NULL;
+	PyObject *ret;
 	PyObject *rmhomedir = NULL, *rmmailspool = NULL;
 	struct lu_context *context;
 	char *keywords[] = { "entity", "rmhomedir", "rmmailspool", NULL };
@@ -1020,7 +1044,7 @@ libuser_admin_enumerate_users(PyObject *self, PyObject *args,
 {
 	GValueArray *results;
 	const char *pattern = NULL;
-	PyObject *ret = NULL;
+	PyObject *ret;
 	struct lu_error *error = NULL;
 	char *keywords[] = { "pattern", NULL };
 	struct libuser_admin *me = (struct libuser_admin *) self;
@@ -1034,9 +1058,12 @@ libuser_admin_enumerate_users(PyObject *self, PyObject *args,
 	}
 	/* Read the list of all users. */
 	results = lu_users_enumerate(me->ctx, pattern, &error);
+	if (error != NULL)
+		lu_error_free(&error);
 	/* Convert the list to a PyList. */
 	ret = convert_value_array_pylist(results);
-	g_value_array_free(results);
+	if (results != NULL)
+		g_value_array_free(results);
 	DEBUG_EXIT;
 	return ret;
 }
@@ -1048,7 +1075,7 @@ libuser_admin_enumerate_groups(PyObject *self, PyObject *args,
 {
 	GValueArray *results;
 	const char *pattern = NULL;
-	PyObject *ret = NULL;
+	PyObject *ret;
 	struct lu_error *error = NULL;
 	char *keywords[] = { "pattern", NULL };
 	struct libuser_admin *me = (struct libuser_admin *) self;
@@ -1062,9 +1089,12 @@ libuser_admin_enumerate_groups(PyObject *self, PyObject *args,
 	}
 	/* Get the list of groups. */
 	results = lu_groups_enumerate(me->ctx, pattern, &error);
+	if (error != NULL)
+		lu_error_free(&error);
 	/* Convert the list to a PyList. */
 	ret = convert_value_array_pylist(results);
-	g_value_array_free(results);
+	if (results != NULL)
+		g_value_array_free(results);
 	DEBUG_EXIT;
 	return ret;
 }
@@ -1076,7 +1106,7 @@ libuser_admin_enumerate_users_by_group(PyObject *self, PyObject *args,
 {
 	GValueArray *results;
 	char *group = NULL;
-	PyObject *ret = NULL;
+	PyObject *ret;
 	struct lu_error *error = NULL;
 	char *keywords[] = { "group", NULL };
 	struct libuser_admin *me = (struct libuser_admin *) self;
@@ -1089,8 +1119,11 @@ libuser_admin_enumerate_users_by_group(PyObject *self, PyObject *args,
 	}
 	/* Get a list of the users in this group. */
 	results = lu_users_enumerate_by_group(me->ctx, group, &error);
+	if (error != NULL)
+		lu_error_free(&error);
 	ret = convert_value_array_pylist(results);
-	g_value_array_free(results);
+	if (results != NULL)
+		g_value_array_free(results);
 	DEBUG_EXIT;
 	return ret;
 }
@@ -1102,7 +1135,7 @@ libuser_admin_enumerate_groups_by_user(PyObject *self, PyObject *args,
 {
 	GValueArray *results;
 	char *user = NULL;
-	PyObject *ret = NULL;
+	PyObject *ret;
 	struct lu_error *error = NULL;
 	char *keywords[] = { "user", NULL };
 	struct libuser_admin *me = (struct libuser_admin *) self;
@@ -1115,8 +1148,11 @@ libuser_admin_enumerate_groups_by_user(PyObject *self, PyObject *args,
 	}
 	/* Get the list. */
 	results = lu_groups_enumerate_by_user(me->ctx, user, &error);
+	if (error != NULL)
+		lu_error_free(&error);
 	ret = convert_value_array_pylist(results);
-	g_value_array_free(results);
+	if (results != NULL)
+		g_value_array_free(results);
 	DEBUG_EXIT;
 	return ret;
 }
@@ -1128,7 +1164,7 @@ libuser_admin_enumerate_users_full(PyObject *self, PyObject *args,
 {
 	GPtrArray *results;
 	const char *pattern = NULL;
-	PyObject *ret = NULL;
+	PyObject *ret;
 	struct lu_error *error = NULL;
 	char *keywords[] = { "pattern", NULL };
 	struct libuser_admin *me = (struct libuser_admin *) self;
@@ -1143,6 +1179,8 @@ libuser_admin_enumerate_users_full(PyObject *self, PyObject *args,
 	}
 	/* Read the list of all users. */
 	results = lu_users_enumerate_full(me->ctx, pattern, &error);
+	if (error != NULL)
+		lu_error_free(&error);
 	/* Convert the list to a PyList. */
 	ret = PyList_New(0);
 	for (i = 0; i < results->len; i++) {
@@ -1164,7 +1202,7 @@ libuser_admin_enumerate_groups_full(PyObject *self, PyObject *args,
 {
 	GPtrArray *results;
 	const char *pattern = NULL;
-	PyObject *ret = NULL;
+	PyObject *ret;
 	struct lu_error *error = NULL;
 	char *keywords[] = { "pattern", NULL };
 	struct libuser_admin *me = (struct libuser_admin *) self;
@@ -1179,6 +1217,8 @@ libuser_admin_enumerate_groups_full(PyObject *self, PyObject *args,
 	}
 	/* Get the list of groups. */
 	results = lu_groups_enumerate_full(me->ctx, pattern, &error);
+	if (error != NULL)
+		lu_error_free(&error);
 	/* Convert the list to a PyList. */
 	ret = PyList_New(0);
 	for (i = 0; i < results->len; i++) {
@@ -1201,7 +1241,7 @@ libuser_admin_enumerate_users_by_group_full(PyObject *self, PyObject *args,
 {
 	GPtrArray *results;
 	char *group = NULL;
-	PyObject *ret = NULL;
+	PyObject *ret;
 	char *keywords[] = { "group", NULL };
 	int i;
 
@@ -1232,7 +1272,7 @@ libuser_admin_enumerate_groups_by_user_full(PyObject *self, PyObject *args,
 {
 	GPtrArray *results;
 	char *user = NULL;
-	PyObject *ret = NULL;
+	PyObject *ret;
 	struct lu_error *error = NULL;
 	char *keywords[] = { "user", NULL };
 	struct libuser_admin *me = (struct libuser_admin *) self;
